<?php

namespace Drupal\monarch_csv_import;

/**
 * Iterates over csv rows.
 */
class CsvIterator implements \Iterator {

  /**
   * Holds the current data.
   *
   * @var array|null
   */
  protected $data;

  /**
   * Holds the header columns.
   *
   * @var array
   */
  protected $header;

  /**
   * Holds the file stream.
   *
   * @var array
   */
  protected $file;

  /**
   * Index for the data.
   *
   * @var int
   */
  protected $index;

  /**
   * The field delimiter.
   *
   * @var string
   */
  protected $separator;

  /**
   * The field enclosure character.
   *
   * @var string
   */
  protected $enclosure;

  /**
   * The escape character.
   *
   * @var string
   */
  protected $escape;

  /**
   * Constructs the CsvIterator.
   *
   * @param string $file_path
   *   The file path or public/private uri for the CSV file.
   * @param string $separator
   *   Set the field delimiter (one character only).
   * @param string $enclosure
   *   Set the field enclosure character (one character only).
   * @param string $escape
   *   Set the escape character (one character only). Defaults as a backslash.
   */
  public function __construct(string $file_path, string $separator = ',', string $enclosure = '"', string $escape = '\\') {
    if (!($this->file = fopen($file_path, 'r'))) {
      throw new \Exception('Unable to open ' . $file_path . ' for reading.');
    }

    $this->separator = $separator;
    $this->enclosure = $enclosure;
    $this->escape = $escape;

    $this->rewind();
  }

  /**
   * {@inheritdoc}
   */
  public function __destruct() {
    if ($this->file) {
      fclose($this->file);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function current() {
    return $this->data;
  }

  /**
   * {@inheritdoc}
   */
  public function key() {
    return $this->index;
  }

  /**
   * Gets the data for the current index.
   */
  protected function getCurrent() {
    $fields = fgetcsv($this->file, 0, $this->separator, $this->enclosure, $this->escape) ?: NULL;
    $data = [];

    if (!empty($fields)) {
      foreach ($fields as $index => $field) {
        if (!isset($this->header[$index]) || empty($this->header[$index])) {
          throw new \Exception('Missing column name for column ' . ($index + 1));
        }

        $data[$this->header[$index]] = $field;
      }

      $this->data = $data;
    }
    else {
      $this->data = NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function next() : void {
    $this->index++;
    $this->getCurrent();
  }

  /**
   * {@inheritdoc}
   */
  public function rewind() : void {
    fseek($this->file, 0);
    $this->header = fgetcsv($this->file, 0, $this->separator, $this->enclosure, $this->escape) ?: NULL;
    $this->index = 0;
    $this->getCurrent();
  }

  /**
   * {@inheritdoc}
   */
  public function valid() : bool {
    return !empty($this->data);
  }

  /**
   * Returns all of the column names in the csv.
   *
   * @return array
   *   The list of column names.
   */
  public function columns() : array {
    return $this->header;
  }

}
